'' =================================================================================================
''
''   File....... jm_pca9554.spin
''   Purpose.... 
''   Author..... Jon "JonnyMac" McPhalen
''               Copyright (c) 2013 Jon McPhalen
''               -- see below for terms of use
''   E-mail..... jon@jonmcphalen.com
''   Started.... 
''   Updated.... 15 APR 2013
''
'' =================================================================================================


con

  RX1 = 31                                                      ' programming / terminal
  TX1 = 30
  
  SDA = 29                                                      ' eeprom / i2c
  SCL = 28


con

  WR_9554 = %0100_000_0
  RD_9554 = %0100_000_1

  R_INS   = 0                                                   ' read input port
  R_OUTS  = 1                                                   ' read/write output port
  R_POL   = 2                                                   ' read/write polarity inversion register
  R_CFG   = 3                                                   ' read/write configuration register
  

obj

  i2c : "jm_i2c"
  

var


pub start(sclpin, sdapin)

'' Start PCA9554 object
'' -- sclpin and sdapin define i2c buss
''    * can be 28 and 29 if use EEPROM buss

  i2c.setupx(sclpin, sdapin)                                    ' connect to buss


pub wr_reg(addr, reg, b)

'' Writes byte b to register in device at addr

  i2c.start
  i2c.write(WR_9554 | (addr << 1))
  i2c.write(reg)
  i2c.write(b)
  i2c.stop


pub rd_reg(addr, reg) | b

'' Read register from device at addr

  i2c.start
  i2c.write(WR_9554 | (addr << 1))
  i2c.write(reg)
  i2c.start
  i2c.write(RD_9554 | (addr << 1))
  b := i2c.read(i2c#ACK) 
  i2c.stop

  return b

  
pub wr_port(addr, b)

'' Write b to port bits of device at addr

   wr_reg(addr, R_OUTS, b)
  

pub rd_port(addr)

'' Read port bits from device at addr

  return rd_reg(addr, R_INS)


pub wr_pol(addr, polbits)

'' Write polbits to polarity inversion register of device at addr
'' -- "1" in polbits inverts corresponding port

  wr_reg(addr, R_POL, polbits)


pub wr_cfg(addr, cfgbits)

'' Write cfgbits to configration register of device at addr
'' -- "1" in cfgbits makes corresponding pin an input
'' -- "0" in cfgbits makes corresponding pin an output

  wr_reg(addr, R_CFG, cfgbits)


dat

{{

  Terms of Use: MIT License

  Permission is hereby granted, free of charge, to any person obtaining a copy of this
  software and associated documentation files (the "Software"), to deal in the Software
  without restriction, including without limitation the rights to use, copy, modify,
  merge, publish, distribute, sublicense, and/or sell copies of the Software, and to
  permit persons to whom the Software is furnished to do so, subject to the following
  conditions:

  The above copyright notice and this permission notice shall be included in all copies
  or substantial portions of the Software.

  THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR IMPLIED,
  INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY, FITNESS FOR A
  PARTICULAR PURPOSE AND NON-INFRINGEMENT. IN NO EVENT SHALL THE AUTHORS OR COPYRIGHT
  HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF
  CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE
  OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

}}